package org.example.sweater.controller;

import jakarta.servlet.http.HttpServletRequest;
import org.example.sweater.domain.Message;
import org.example.sweater.domain.User;
import org.example.sweater.repos.MessagesRepository;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.web.csrf.CsrfToken;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

@Controller
public class MainController {
    public MainController(MessagesRepository messagesRepository) {
        this.messagesRepository = messagesRepository;
    }

    private final MessagesRepository messagesRepository;

    @Value("${upload.path}")
    private String uploadPath;

    @ModelAttribute
    public void addCsrfToken(Model model, HttpServletRequest request) {
        CsrfToken token = (CsrfToken) request.getAttribute(CsrfToken.class.getName());
        model.addAttribute("_csrf", token);
    }

    @GetMapping("/")
    public String mainPage() {
        return "main";
    }

    @GetMapping("/feed")
    public String feed(
            @RequestParam(required = false)
            String tag,
            @RequestParam(required = false)
            String prefix,
            Model model
    ) {
        Iterable<Message> messages;

        if (prefix != null && !prefix.isEmpty()) {
            messages = messagesRepository.findByTextStartingWith(prefix);
            model.addAttribute("search", prefix);
        } else if (tag != null && !tag.isEmpty()) {
            messages = messagesRepository.findByTag(tag);
            model.addAttribute("tag", tag);
        } else {
            messages = messagesRepository.findAll();
        }

        model.addAttribute("messages", messages);

        return "feed";
    }

    @PostMapping("/feed")
    public String addMessage(
            @AuthenticationPrincipal User user,
            @RequestParam String text,
            @RequestParam String tag,
            @RequestParam("file") MultipartFile file,
            Map<String, Object> model
    ) throws IOException {
        Message createdMessage = new Message(text, tag, user);

        if (file != null && !Objects.requireNonNull(file.getOriginalFilename()).isEmpty()) {
            File uploadDir = new File(uploadPath);

            if (!uploadDir.exists()) {
                uploadDir.mkdir();
            }

            String uuidFile = UUID.randomUUID().toString();
            String fileName = uuidFile + "_" + file.getOriginalFilename();

            file.transferTo(new File(uploadDir.getAbsolutePath() + "/" + fileName));

            createdMessage.setFilename(fileName);
        }

        messagesRepository.save(createdMessage);

        Iterable<Message> allMessages = messagesRepository.findAll();
        model.put("messages", allMessages);

        return "feed";
    }

    @PostMapping("/feed/delete")
    public String removeMessageById(
            @RequestParam String id,
            Map<String, Object> model
    ) {
        Long idAsInt = Long.parseLong(id);
        messagesRepository.deleteById(idAsInt);

        Iterable<Message> foundMessages = messagesRepository.findAll();

        model.put("messages", foundMessages);

        return "feed";
    }
}
