package org.example.sweater.controller;

import org.example.sweater.domain.Role;
import org.example.sweater.domain.User;
import org.example.sweater.repos.UsersRepository;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/user")
@PreAuthorize("hasAuthority('ADMIN')")
public class UserController {
    public final UsersRepository usersRepository;
    public UserController(UsersRepository usersRepository) {
        this.usersRepository = usersRepository;
    }

    @GetMapping
    public String userList(Model model) {
        Collection<User> allUsers = usersRepository.findAll();
        model.addAttribute("users", allUsers);

        return "userList";
    }

    @GetMapping("{user}")
    public String userEdit(
            @PathVariable("user") User user,
            Model model
    ) {
        model.addAttribute("user", user);

        Role[] allRoles = Role.values();
        model.addAttribute("roles", allRoles);

        return "userEdit";
    }

    @PostMapping
    public String userUpdate(
            @RequestParam("userId") User user,
            @RequestParam("userName") String username,
            @RequestParam Map<String, String> formData
    ) {
        user.setUsername(username);
        Set<String> rolesAsStrings = Arrays
                .stream(Role.values())
                .map(Role::name)
                .collect(Collectors.toSet());

        user.getRoles().clear();

        for (String key : formData.keySet()) {
            if (!rolesAsStrings.contains(key)) {
                continue;
            }
            user.getRoles().add(Role.valueOf(key));
        }

        usersRepository.save(user);
        return "redirect:/user";
    }
}
